/* Nessus
 * Copyright (C) 1998 - 2001 Renaud Deraison
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * as published by the Free Software Foundation
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * In addition, as a special exception, Renaud Deraison
 * gives permission to link the code of this program with any
 * version of the OpenSSL library which is distributed under a
 * license identical to that listed in the included COPYING.OpenSSL
 * file, and distribute linked combinations including the two.
 * You must obey the GNU General Public License in all respects
 * for all of the code used other than OpenSSL.  If you modify
 * this file, you may extend this exception to your version of the
 * file, but you are not obligated to do so.  If you do not wish to
 * do so, delete this exception statement from your version.
 *
 */ 
 
#include <includes.h>
#include "processes.h"
#include "sighand.h"
#include "log.h"




static int process_son = 0;

void sighand_process_term(int sig)
{
 int son = process_son;
 if(son){
 	terminate_process(son);
	process_son = 0;
	}
 _EXIT(0);
}



static void pr_sigterm(int sig)
{
 _exit(0);
}


int terminate_process(pid_t pid)
{
 return kill(pid, SIGTERM);
}




/*
 * Create a thread
 */
pid_t
create_process(function, argument)
  process_func_t function;
  void * argument;
{
 int pid;

 

 pid = fork();

 if (pid == 0)
 { 
  process_son = 0;
  nessus_signal(SIGHUP, SIG_IGN);
  nessus_signal(SIGTERM, pr_sigterm);
  nessus_signal(SIGINT, pr_sigterm);
  nessus_signal(SIGPIPE, SIG_IGN);
  nessus_signal(SIGUSR1, SIG_IGN);
  nessus_signal(SIGUSR2, SIG_IGN);
  nessus_signal(SIGSEGV, sighand_segv);	/* Comment this line out to dump a core and debug nessusd */
  srand48((long)time(NULL));
  (*function)(argument);
  EXIT(0);
 }
 if(pid < 0)
 	log_write("Error : could not fork ! Error : %s\n", strerror(errno));
 process_son = pid;
 return pid;
} 



